import React from "react";
import { mixColour } from "../utils/colour";
import { Child } from "../types";
import * as SC from "@dothq/styled-components";

export interface FoundationTheme {
    colours: {
        [key: string]: {
            [key: number]: string
        }
    },
    fonts: {
        bodyFont: string,
        sans: string[],
        [key: string]: string[] | string
    }
}

export const defaultTheme: FoundationTheme = {
    colours: {
        primary: {
            100: mixColour("#1662D3", -44),
            200: mixColour("#1662D3", -33),
            300: mixColour("#1662D3", -22),
            400: mixColour("#1662D3", -11),
            /** Base colour */ 500: mixColour("#1662D3", 0), 
            600: mixColour("#1662D3", 11),
            700: mixColour("#1662D3", 22),
            800: mixColour("#1662D3", 33),
            900: mixColour("#1662D3", 44)
        }
    },
    fonts: {
        bodyFont: "sans",
        sans: [
            "-apple-system",
            "BlinkMacSystemFont",
            '"Segoe UI"',
            "Helvetica",
            "Arial",
            "sans-serif",
            '"Apple Color Emoji"',
            '"Segoe UI Emoji"'
        ]
    }
};

export const ThemeProvider = ({ 
    theme, 
    css,
    children
}: { 
    theme?: FoundationTheme, 
    css?: string,
    children: Child
}) => {
    const mergedTheme = { ...defaultTheme, ...theme };

    const GlobalStyle = SC.createGlobalStyle`
        ${css || ""};

        :root {
            ${Object.entries(mergedTheme.fonts).map(([key, val]) => {
                if(!Array.isArray(val)) return;

                return `--fdn-${key}-font: var(--fdn-override-${key}-font, ${val.join(", ")})`
            }).join("\n")}
        }

        html,
        body {
            font-family: var(--fdn-${mergedTheme.fonts.bodyFont}-font);
        }
    `;

    return (
        <SC.ThemeProvider 
            theme={mergedTheme}
        >
            <>
                <GlobalStyle />
                {children}
            </>
        </SC.ThemeProvider>
    )
}